@@ -23,6 +23,10 @@
 #'
 #' - `extract_recipe()` returns a recipe, possibly estimated.
 #'
+#' - `extract_postprocessor()` returns the post-processor.
+#'
+#' - `extract_tailor()` returns a tailor, possibly fit.
+#'
 #' - `extract_workflow()` returns a workflow, possibly fit.
 #'
 #' - `extract_parameter_dials()` returns a single dials parameter object.
@@ -42,6 +46,8 @@
 #' # `workflows`.
 NULL
 
+# nocov start
+
 #' @rdname hardhat-extract
 #' @export
 extract_workflow <- function(x, ...) {
@@ -72,7 +78,6 @@ extract_fit_engine <- function(x, ...) {
   UseMethod("extract_fit_engine")
 }
 
-
 #' @rdname hardhat-extract
 #' @export
 extract_mold <- function(x, ...) {
@@ -91,6 +96,12 @@ extract_postprocessor <- function(x, ...) {
   UseMethod("extract_postprocessor")
 }
 
+#' @rdname hardhat-extract
+#' @export
+extract_tailor <- function(x, ...) {
+  UseMethod("extract_tailor")
+}
+
 #' @rdname hardhat-extract
 #' @export
 extract_parameter_dials <- function(x, ...) {
@@ -108,3 +119,5 @@ extract_parameter_set_dials <- function(x, ...) {
 extract_fit_time <- function(x, ...) {
   UseMethod("extract_fit_time")
 }
+
+# nocov end
