@@ -11,6 +11,8 @@
 #' typically the tibble returned from the corresponding call to
 #' [model_frame()].
 #'
+#' @inheritParams validate_column_names
+#'
 #' @details
 #'
 #' The following explains the rationale for some of the difference in arguments
@@ -77,23 +79,24 @@
 #'   contrasts = global_override
 #' )
 #' @export
-model_matrix <- function(terms, data) {
-  validate_is_terms(terms)
-  data <- check_is_data_like(data)
+model_matrix <- function(terms, data, ..., call = current_env()) {
+  check_dots_empty0(...)
+  check_terms(terms, call = call)
+  check_data_frame_or_matrix(data, call = call)
+  data <- coerce_to_tibble(data)
 
   # otherwise model.matrix() will try and run model.frame() for us on data
   # but we definitely don't want this, as we have already done it and it can
   # actually error out if we don't prevent it from running
   attr(data, "terms") <- terms
 
-  predictors <- with_options(
-    model.matrix(object = terms, data = data),
-    na.action = "na.pass"
+  predictors <- with_na_pass(
+    model.matrix(object = terms, data = data)
   )
 
   predictors <- strip_model_matrix(predictors)
 
-  tibble::as_tibble(predictors)
+  tibble::as_tibble(predictors, .name_repair = "minimal")
 }
 
 strip_model_matrix <- function(x) {
@@ -109,23 +112,29 @@ strip_model_matrix <- function(x) {
   x
 }
 
-is_terms <- function(x) {
-  inherits(x, "terms")
-}
-
-validate_is_terms <- function(.x, .x_nm) {
-  if (is_missing(.x_nm)) {
-    .x_nm <- as_label(enexpr(.x))
-  }
-
-  validate_is(.x, is_terms, "terms object", .x_nm)
+check_terms <- function(
+  x,
+  ...,
+  allow_null = FALSE,
+  arg = caller_arg(x),
+  call = caller_env()
+) {
+  check_inherits(
+    x = x,
+    what = "terms",
+    allow_null = allow_null,
+    arg = arg,
+    call = call
+  )
 }
 
 # ------------------------------------------------------------------------------
 
-model_matrix_one_hot <- function(terms, data) {
-  validate_is_terms(terms)
-  data <- check_is_data_like(data)
+model_matrix_one_hot <- function(terms, data, ..., call = caller_env()) {
+  check_dots_empty0(...)
+  check_terms(terms, call = call)
+  check_data_frame_or_matrix(data, call = call)
+  data <- coerce_to_tibble(data)
 
   n_cols <- length(data)
 
@@ -156,7 +165,7 @@ model_matrix_one_hot <- function(terms, data) {
     data[[name]] <- assign_contrasts(col, n, contrasts)
   }
 
-  model_matrix(terms, data)
+  model_matrix(terms, data, call = call)
 }
 
 #' Contrast function for one-hot encodings
@@ -164,37 +173,49 @@ model_matrix_one_hot <- function(terms, data) {
 #' This contrast function produces a model matrix that has indicator columns for
 #' each level of each factor.
 #'
-#' @param n A vector of character factor levels or the number of unique levels.
+#' @param n A vector of character factor levels (of length >=1) or the number
+#' of unique levels (>= 1).
 #' @param contrasts This argument is for backwards compatibility and only the
 #'   default of `TRUE` is supported.
 #' @param sparse This argument is for backwards compatibility and only the
 #'   default of `FALSE` is supported.
 #'
+#' @includeRmd man/rmd/one-hot.Rmd details
+#'
 #' @return A diagonal matrix that is `n`-by-`n`.
 #'
 #' @keywords internal
+#' @export
 contr_one_hot <- function(n, contrasts = TRUE, sparse = FALSE) {
   if (sparse) {
-    warn("`sparse = TRUE` not implemented for `contr_one_hot()`.")
+    cli::cli_warn(
+      "{.code sparse = TRUE} not implemented for {.fun contr_one_hot}."
+    )
   }
 
   if (!contrasts) {
-    warn("`contrasts = FALSE` not implemented for `contr_one_hot()`.")
+    cli::cli_warn(
+      "{.code contrasts = FALSE} not implemented for {.fun contr_one_hot}."
+    )
   }
 
   if (is.character(n)) {
+    if (length(n) < 1) {
+      cli::cli_abort("{.arg n} cannot be empty.")
+    }
     names <- n
     n <- length(names)
   } else if (is.numeric(n)) {
+    check_number_whole(n, min = 1)
     n <- as.integer(n)
 
     if (length(n) != 1L) {
-      abort("`n` must have length 1 when an integer is provided.")
+      cli::cli_abort("{.arg n} must have length 1 when an integer is provided.")
     }
 
     names <- as.character(seq_len(n))
   } else {
-    abort("`n` must be a character vector or an integer of size 1.")
+    check_number_whole(n, min = 1)
   }
 
   out <- diag(n)
