@@ -9,6 +9,8 @@
 #'
 #' @param data A data frame or matrix.
 #'
+#' @inheritParams validate_column_names
+#'
 #' @return
 #'
 #' A 0 row slice of `data` after converting it to a tibble.
@@ -22,16 +24,17 @@
 #' @examples
 #'
 #' hardhat:::extract_ptype(iris)
-#'
 #' @keywords internal
 #'
-extract_ptype <- function(data) {
+extract_ptype <- function(data, ..., call = current_env()) {
+  check_dots_empty0(...)
 
   if (is.null(data)) {
     return(NULL)
   }
 
-  data <- check_is_data_like(data)
+  check_data_frame_or_matrix(data, call = call)
+  data <- coerce_to_tibble(data)
 
-  vctrs::vec_slice(data, 0L)
+  vec_slice(data, 0L)
 }
